using System;
using System.Runtime.InteropServices;
using System.Web.UI;
using System.Reflection;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Xml.Serialization;
using System.ComponentModel;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;
using Microsoft.SharePoint.WebPartPages;
using System.Text;


namespace MultiLingualPost
{
    /// <summary>
    /// History
    /// -----------------------------------
    /// Author : Stphane Eyskens
    /// Creation Date: 2007 08 14
    /// Last Revision Date: 2007 08 14
    /// -----------------------------------
    /// Purpose:
    /// -----------------------------------
    /// The purpose of this webpart is to replace the standard "Posts" webpart available in the blogs. The standard webpart doesn't handle multiple
    /// languages in a single blog. Adding a metadata column will just make your webpart rendering differently because you need to associate it with
    /// a view and by doing that, you lose the standard display.
    /// 
    /// This webpart just keeps the standard way of displaying posts and adds the possibility of creating a metadata column to store the language and then
    /// to filter posts on that column later on. It has been created for fun and is subject to changes since I plan to use it in my professional blog.
    /// 
    /// </summary>
    [Guid("8d2a835f-e786-413c-8bc0-6169b4cd7d4a")]
    [DefaultProperty("Text"),
ToolboxData("<{0}:MultiLingualPost runat=server></{0}:MultiLingualPost>"),
   XmlRoot(Namespace = "MultiLingualPost")]
    public class MultiLingualPost : Microsoft.SharePoint.WebPartPages.WebPart
    {
        #region members
        private IWebPartField LanguageProvider;
        private object ProvidedLanguageValue;
        public Label ErrorEncountered = null;
        public HtmlTable BodyTable = null;
        public bool _ExpandedEnabled;
        public string _LanguageField = null;
        #endregion

        #region properties
        [Browsable(false),
        Category("Post Settings"),
        WebPartStorage(Storage.Personal),
        FriendlyName("Language Field"),
        Description("Select the column to apply")]
        public string LanguageField
        {
            get
            {
                return _LanguageField;
            }
            set
            {
                _LanguageField = value;
            }
        }

        [Category("Post Settings")]
        [WebPartStorage(Storage.Personal)]
        [FriendlyNameAttribute("Expand/Collapse enabled")]
        [Description("Check that checkbox to enable the expanding/collapsing feature")]
        [Browsable(true)]
        [XmlElement(ElementName = "ExpandedEnabled")]
        public bool ExpandedEnabled
        {
            get
            {
                return _ExpandedEnabled;
            }
            set
            {
                _ExpandedEnabled = value;
            }
        }
#endregion

        /// <summary>
        /// default empty constructor
        /// </summary>
        public MultiLingualPost()
        {                     
           
        }
        /// <summary>
        /// Gets the value transmitted by the provider webpart
        /// </summary>
        /// <param name="fieldValue"></param>
        private void GetFieldValue(object fieldValue)
        {
            ProvidedLanguageValue = fieldValue;
        }
        /// <summary>
        /// This method is called by the webpartmanager whenever a connection is established to our webpart
        /// </summary>
        /// <param name="provider"></param>
        [ConnectionConsumer("FieldConsumer", "LanguageConnectionPoint",
      typeof(FieldConsumerConnectionPoint), AllowsMultipleConnections = false)]
        public void SetConnectionInterface(IWebPartField provider)
        {
            LanguageProvider = provider;
        }
        /// <summary>
        /// Adding the main body table and an error label.
        /// </summary>
        protected override void CreateChildControls()
        {
            ErrorEncountered = new Label();
            ErrorEncountered.Style.Add(HtmlTextWriterStyle.Color, "red");
            BodyTable = new HtmlTable();

            this.Controls.Add(BodyTable);
            this.Controls.Add(ErrorEncountered);
        }
        /// <summary>
        /// This webpart exposes one dropdown list containing the names of the fields belonging to the post column. This
        /// is to specify which column stores the language information
        /// </summary>
        /// <returns></returns>
        public override ToolPart[] GetToolParts()
        {
            ToolPart[] toolparts = new ToolPart[3];
            WebPartToolPart wptp = new WebPartToolPart();
            CustomPropertyToolPart custom = new CustomPropertyToolPart();
            toolparts[0] = custom;
            toolparts[1] = wptp;
            // This is the custom ToolPart.
            toolparts[2] = new FieldList();
            return toolparts;

        }
        /// <summary>
        /// If a connection is established, make sure to get the value from the provider
        /// </summary>
        /// <param name="e"></param>
        protected override void OnPreRender(EventArgs e)
        {
            if (LanguageProvider != null)
            {
                LanguageProvider.GetFieldValue(new FieldCallback(GetFieldValue));
            }
            base.OnPreRender(e);
        }

        /// <summary>
        /// Renders the webpart
        /// </summary>
        /// <param name="writer"></param>
        protected override void Render(HtmlTextWriter writer)
        {                      
            

            SPList PostList = null;

            try
            {
                //Make sure we're in the context of a blog
                if (SPContext.Current.Web.WebTemplate != SPWebTemplate.WebTemplateBLOG)
                {
                    throw new ApplicationException("This webpart can only be used inside blogs");
                }
                PostList = SPContext.Current.Web.Lists["Posts"];
                SPQuery PostQuery = new SPQuery();
                //that query retrieves all the items sorted descendingly on the creation date
                PostQuery.Query = "<OrderBy><FieldRef Name=\"Created\" Ascending=\"FALSE\" /></OrderBy>";

                if (LanguageProvider != null)
                {
                    if (LanguageField != null)
                    {
                        PropertyDescriptor prop = LanguageProvider.Schema;
                        //If the value provided value isn't null, I create an SPQuery that takes the provided value into account.
                        if (prop != null && ProvidedLanguageValue != null && ProvidedLanguageValue.ToString().ToLower() != "no filter")
                        {
                            StringBuilder SpQueryString = new StringBuilder();
                            SpQueryString.Append("<OrderBy><FieldRef Name=\"Created\" Ascending=\"FALSE\" /></OrderBy>");
                            SpQueryString.AppendFormat("<Where><Eq><FieldRef Name='{0}' /><Value Type='Text'>{1}</Value></Eq></Where>",
                                PostList.Fields[LanguageField].InternalName,//Must use the internal name in CAML queries
                                ProvidedLanguageValue //contains the value provided by the provider   
                                );
                            PostQuery.Query = SpQueryString.ToString();
                        }                        
                    }
                    else
                    {
                        throw new ApplicationException("Please specify the language field with the toolpane");
                    }
                }  
                //The viewfields property tells the system which fields to include in the result set.
                PostQuery.ViewFields = "<FieldRef Name='Title' /><FieldRef Name='PostCategory' /><FieldRef Name='NumComments' /><FieldRef Name='Body' /><FieldRef Name='EmailPostLink' /><FieldRef Name='PostedByWithDate' />";
                //Fetching the resultset
                foreach (SPListItem PostItem in PostList.GetItems(PostQuery))
                {
                    HtmlTable ItemTable = new HtmlTable();
                    HtmlTableRow ItemRow = new HtmlTableRow();
                    HtmlTableCell ItemCell = new HtmlTableCell();
                    
                    HtmlTableRow TitleRow = new HtmlTableRow();
                    
                    HtmlTableCell TitleCell = new HtmlTableCell();
                    TitleCell.Attributes.Add("class", "ms-PostTitle");
                    StringBuilder TitleCellHtml = new StringBuilder();
                    if (ExpandedEnabled)
                    {
                        //If the expanding/collapsing functionality has been activated, I just add an image next to each post title
                        TitleCellHtml.Append("<img onclick=\"if(this.src.indexOf('minus') != -1){");
                        TitleCellHtml.Append("this.parentNode.parentNode.parentNode.rows[1].style.display='none';");
                        TitleCellHtml.Append("this.src='_layouts/images/plus.gif';}");
                        TitleCellHtml.Append("else{this.parentNode.parentNode.parentNode.rows[1].style.display='';");
                        TitleCellHtml.Append("this.src='_layouts/images/minus.gif';}\" src='_layouts/images/minus.gif'/>&nbsp;");
                    }
                    TitleCellHtml.AppendFormat("<a style='font-family:verdana;font-size:26px;' href='{0}/Lists/Posts/Post.aspx?ID={1}'>{2}</a>",
                        SPContext.Current.Web.ServerRelativeUrl,
                        PostItem["ID"].ToString(),
                        PostItem["Title"].ToString());

                    TitleCell.InnerHtml = TitleCellHtml.ToString();
                    HtmlTableRow BodyRow = new HtmlTableRow();
                    HtmlTableCell BodyCell = new HtmlTableCell();
                    BodyCell.Attributes.Add("class", "ms-PostBody");
                    BodyCell.InnerHtml = PostItem["Body"].ToString();
                    BodyRow.Cells.Add(BodyCell);
                    HtmlTableRow BottomRow = new HtmlTableRow();
                    HtmlTableCell BottomCell = new HtmlTableCell();
                    StringBuilder BottomCellHtml = new StringBuilder();
                    //This cell is displayed at the bottom of the post to inform users about who posted what and when
                    BottomCellHtml.AppendFormat("Posted by {0} at {1} | <a href='{2}/Lists/Posts/Post.aspx?ID={3}#Comments'>Comments({4})</a>",
                        PostItem["PostedByWithDate"],
                        PostItem["Created"],
                        SPContext.Current.Web.ServerRelativeUrl,
                        PostItem["ID"].ToString(),
                        PostItem["NumComments"]);

                    BottomCell.InnerHtml = BottomCellHtml.ToString();
                    BottomRow.Cells.Add(BottomCell);
                    TitleRow.Cells.Add(TitleCell);
                    ItemTable.Rows.Add(TitleRow);
                    ItemTable.Rows.Add(BodyRow);
                    ItemTable.Rows.Add(BottomRow);
                    ItemCell.Controls.Add(ItemTable);
                    ItemRow.Cells.Add(ItemCell);
                    BodyTable.Rows.Add(ItemRow);
                    
                }
                BodyTable.RenderControl(writer);
            }
            catch (ArgumentException ArgEx)
            {
                ErrorEncountered.Text = ArgEx.Message;
            }
            catch (ApplicationException AppEx)
            {
                ErrorEncountered.Text = AppEx.Message;
            }
            catch (Exception GenericEx)
            {
                ErrorEncountered.Text = GenericEx.Message;
            }            
            if (ErrorEncountered.Text != "")
            {
                ErrorEncountered.RenderControl(writer);
            }
        }
    }

    /// T<summary>
    /// This class handles the connection
    /// </summary>
    public class FieldConsumerConnectionPoint : ConsumerConnectionPoint
    {
        public FieldConsumerConnectionPoint(MethodInfo callbackMethod,
          Type interfaceType, Type controlType, string name, string id,
          bool allowsMultipleConnections)
            : base(
          callbackMethod, interfaceType, controlType,
          name, id, allowsMultipleConnections)
        {
        }

        
    }
}
